# -*- coding: utf-8 -*-
# @Time    : 2018/8/1 9:48
# @Author  : yangyong
# @Email   : yangyong@findourlove.com
# @File    : __init__.py
# @Software: PyCharm
import sys
if sys.version_info[0] == 2:
    str_ = unicode
elif sys.version_info[0] == 3:
    str_ = str


# -*- coding: utf-8 -*-
# @Time    : 2018/8/1 9:35
# @Author  : yangyong
# @Email   : yangyong@findourlove.com
# @File    : __init__.py
# @Software: PyCharm
from __future__ import division, print_function, unicode_literals
import io
import os

try:
    import re2 as re
except ImportError:
    import re
import sys
import math
import getopt
import subprocess
import collections
from .langdata import LANGDATA

if sys.version[0] >= '3':
    unicode = str  # pylint: disable=invalid-name,redefined-builtin

WORDRE = re.compile(r"\b[-\w]+\b", re.UNICODE)
PARARE = re.compile('\n\n+')
SENTRE = re.compile('[^\n]+(\n|$)')


def getmeasures(text, lang='en', merge=False):
    characters = 0
    words = 0
    syllables = 0
    complex_words = 0
    complex_words_dc = 0
    long_words = 0
    paragraphs = 0
    sentences = 0
    vocabulary = set()
    syllcounter = LANGDATA[lang]['syllables']
    wordusageregexps = LANGDATA[lang]['words']
    beginningsregexps = LANGDATA[lang]['beginnings']
    basicwords = LANGDATA[lang].get('basicwords', frozenset())

    wordusage = collections.OrderedDict([(name, 0) for name, regexp
                                         in wordusageregexps.items()])
    beginnings = collections.OrderedDict([(name, 0) for name, regexp
                                          in beginningsregexps.items()])

    if isinstance(text, bytes):
        raise ValueError('Expected: unicode string or an iterable of lines')
    elif isinstance(text, unicode):
        # Collect surface characteristics from a string.
        # NB: only recognizes UNIX newlines.
        paragraphs = sum(1 for _ in PARARE.finditer(text)) + 1
        sentences = sum(1 for _ in SENTRE.finditer(text))
        # paragraphs = text.count('\n\n')
        # sentences = text.count('\n') - paragraphs
        for token in WORDRE.findall(text):
            vocabulary.add(token)
            words += 1
            characters += len(token)
            syll = syllcounter(token)
            syllables += syll
            if len(token) >= 7:
                long_words += 1

            if syll >= 3 and not token[0].isupper():  # ignore proper nouns
                complex_words += 1
            if token.lower() not in basicwords:
                complex_words_dc += 1

        for name, regexp in wordusageregexps.items():
            wordusage[name] += sum(1 for _ in regexp.finditer(text))
        for name, regexp in beginningsregexps.items():
            beginnings[name] += sum(1 for _ in regexp.finditer(text))
    else:  # Collect surface characteristics from an iterable.
        prevempty = True
        for sent in text:
            sent = sent.strip()

            if prevempty and sent:
                paragraphs += 1
            elif not sent:
                prevempty = True
                continue
            prevempty = False

            sentences += 1
            for token in WORDRE.findall(sent):
                vocabulary.add(token)
                words += 1
                characters += len(token)
                syll = syllcounter(token)
                syllables += syll
                if len(token) >= 7:
                    long_words += 1

                if syll >= 3 and not token[0].isupper():  # ignore proper nouns
                    complex_words += 1
                if token.lower() not in basicwords:
                    complex_words_dc += 1

            for name, regexp in wordusageregexps.items():
                wordusage[name] += sum(1 for _ in regexp.finditer(sent))
            for name, regexp in beginningsregexps.items():
                beginnings[name] += regexp.match(sent) is not None

    if not words:
        raise ValueError("I can't do this, there's no words there!")

    stats = collections.OrderedDict([
        ('characters_per_word', characters / words),
        ('syll_per_word', syllables / words),
        ('words_per_sentence', words / sentences),
        ('sentences_per_paragraph', sentences / paragraphs),
        ('type_token_ratio', len(vocabulary) / words),
        ('characters', characters),
        ('syllables', syllables),
        ('words', words),
        ('wordtypes', len(vocabulary)),
        ('sentences', sentences),
        ('paragraphs', paragraphs),
        ('long_words', long_words),
        ('complex_words', complex_words),
    ])
    readability = collections.OrderedDict([
        ('Kincaid', KincaidGradeLevel(syllables, words, sentences)),
        ('ARI', ARI(characters, words, sentences)),
        ('Coleman-Liau',
         ColemanLiauIndex(characters, words, sentences)),
        ('FleschReadingEase',
         FleschReadingEase(syllables, words, sentences)),
        ('GunningFogIndex',
         GunningFogIndex(words, complex_words, sentences)),
        ('LIX', LIX(words, long_words, sentences)),
        ('SMOGIndex', SMOGIndex(complex_words, sentences)),
        ('RIX', RIX(long_words, sentences)),
    ])
    if basicwords:
        stats['complex_words_dc'] = complex_words_dc
        readability['DaleChallIndex'] = DaleChallIndex(
            words, complex_words_dc, sentences)
    if merge:
        readability.update(stats)
        readability.update(wordusage)
        readability.update(beginnings)
        return readability
    return collections.OrderedDict([
        ('readability grades', readability),
        ('sentence info', stats),
        ('word usage', wordusage),
        ('sentence beginnings', beginnings),
    ])


def getdataframe(filenames, lang='en', encoding='utf8', tokenizer=None):
    import pandas
    filenames = list(filenames)

    return pandas.DataFrame([getmeasures(
        applytokenizer(name, tokenizer, encoding),
        lang=lang,
        merge=True)
        for name in filenames], index=filenames)


def applytokenizer(filename, tokenizer, encoding):
    """Run the tokenizer command on a file, if given, and return text."""
    if tokenizer is None:
        return io.open(filename, encoding=encoding).read()
    proc = subprocess.Popen(
        tokenizer.split(),
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE)
    out, _err = proc.communicate(open(filename).read())
    return out.decode(encoding)


def KincaidGradeLevel(syllables, words, sentences):
    return 11.8 * (syllables / words) + 0.39 * ((words / sentences)) - 15.59


def ARI(characters, words, sentences):
    return 4.71 * (characters / words) + 0.5 * (words / sentences) - 21.43


def ColemanLiauIndex(characters, words, sentences):
    return (5.879851 * characters / words - 29.587280 * sentences / words
            - 15.800804)


def FleschReadingEase(syllables, words, sentences):
    return 206.835 - 84.6 * (syllables / words) - 1.015 * (words / sentences)


def GunningFogIndex(words, complex_words, sentences):
    return 0.4 * (((words / sentences)) + (100 * (complex_words / words)))


def LIX(words, long_words, sentences):
    return words / sentences + (100 * long_words) / words


def SMOGIndex(complex_words, sentences):
    return math.sqrt(complex_words * (30 / sentences)) + 3


def RIX(long_words, sentences):
    return long_words / sentences


def DaleChallIndex(words, complex_words_dc, sentences):
    complex_prc = complex_words_dc / words * 100
    if complex_prc <= 5:
        return 0
    return 0.1579 * complex_prc + 0.0496 * words / sentences + 3.6365


def main():
    shortoptions = 'hL:'
    options = 'help csv lang= tokenizer='.split()
    cmd = os.path.basename(sys.argv[0])
    usage = __doc__ % dict(cmd=cmd, lang=', '.join(LANGDATA))
    try:
        opts, args = getopt.gnu_getopt(sys.argv[1:], shortoptions, options)
    except getopt.GetoptError as err:
        print('error: %r\n%s' % (err, usage))
        sys.exit(2)
    opts = dict(opts)
    lang = opts.get('--lang', opts.get('-L', 'en'))

    if '--help' in opts or '-h' in opts:
        print(usage)
        return
    elif '--csv' in opts:
        result = getdataframe(args, lang=lang,
                              tokenizer=opts.get('--tokenizer'))
        result.to_csv(sys.stdout)
        return
    elif len(args) == 0 or args == ['-']:
        text = io.TextIOWrapper(sys.stdin.buffer, encoding='utf8')
    elif len(args) == 1:
        text = applytokenizer(args[0], opts.get('--tokenizer'), 'utf8')
    else:
        raise ValueError('expected 0 or 1 file argument.')
    try:
        for cat, data in getmeasures(text, lang).items():
            print('%s:' % cat)
            for key, val in data.items():
                print(('    %-25s %12.2f' % (key + ':', val)
                       ).rstrip('0 ').rstrip('.'))
    except KeyboardInterrupt:
        sys.exit(1)


__all__ = ['getmeasures', 'getdataframe']

if __name__ == "__main__":  # test unit
    main()



# -*- coding: utf-8 -*-
# @Time    : 2018/8/1 9:44
# @Author  : yangyong
# @Email   : yangyong@findourlove.com
# @File    : langdata.py
# @Software: PyCharm
from __future__ import unicode_literals
import re
import collections

VOWELS = 'aoeuiäàâáåãëéèêóòöôõðùúüìíïî'  # y is special case; true for en.

specialsyllables_en = """\
tottered 2
chummed 1
peeped 1
moustaches 2
shamefully 3
messieurs 2
satiated 4
sailmaker 4
sheered 1
disinterred 3
propitiatory 6
bepatched 2
particularized 5
caressed 2
trespassed 2
sepulchre 3
flapped 1
hemispheres 3
pencilled 2
motioned 2
poleman 2
slandered 2
sombre 2
etc 4
sidespring 2
mimes 1
effaces 2
mr 2
mrs 2
ms 1
dr 2
st 1
sr 2
jr 2
truckle 2
foamed 1
fringed 2
clattered 2
capered 2
mangroves 2
suavely 2
reclined 2
brutes 1
effaced 2
quivered 2
h'm 1
veriest 3
sententiously 4
deafened 2
manoeuvred 3
unstained 2
gaped 1
stammered 2
shivered 2
discoloured 3
gravesend 2
60 2
lb 1
unexpressed 3
greyish 2
unostentatious 5
"""

fallback_cache = {}
_fallback_subsyl = ["cial", "tia", "cius", "cious", "gui", "ion", "iou",
		"sia$", ".ely$"]
_fallback_addsyl = ["ia", "riet", "dien", "iu", "io", "ii",
		"[aeiouy]bl$", "mbl$",
		"[aeiou]{3}",
		"^mc", "ism$",
		"(.)(?!\\1)([aeiouy])\\2l$",
		"[^l]llien",
		"^coad.", "^coag.", "^coal.", "^coax.",
		"(.)(?!\\1)[gq]ua(.)(?!\\2)[aeiou]",
		"dnt$"]
fallback_subsyl = [re.compile(a) for a in _fallback_subsyl]
fallback_addsyl = [re.compile(a) for a in _fallback_addsyl]


def _normalize_word(word):
	return word.strip().lower()

# Read syllable overrides and populate cache with them
for line in specialsyllables_en.splitlines():
	line = line.strip()
	if line:
		toks = line.split()
		assert len(toks) == 2
		fallback_cache[_normalize_word(toks[0])] = int(toks[1])


def countsyllables_en(word):
	"""Fallback syllable counter.

	This is based on the algorithm in Greg Fast's perl module
	Lingua::EN::Syllable."""
	if not word:
		return 0

	# Check for a cached syllable count
	if word in fallback_cache:
		return fallback_cache[word]

	# Remove final silent 'e'
	if word[-1] == "e":
		word = word[:-1]

	# Count vowel groups
	result = 0
	prev_was_vowel = False
	for char in word:
		is_vowel = char in VOWELS or char == 'y'
		if is_vowel and not prev_was_vowel:
			result += 1
		prev_was_vowel = is_vowel

	# Add & subtract syllables
	for r in fallback_addsyl:
		if r.search(word):
			result += 1
	for r in fallback_subsyl:
		if r.search(word):
			result -= 1

	# Cache the syllable count
	fallback_cache[word] = result

	return result


def countsyllables_nlde(word):
	"""Count syllables for Dutch / German words by counting vowel-consonant or
	consonant-vowel pairs, depending on the first character being a vowel or
	not. If it is, a trailing e will be handled with a special rule."""
	result = 0
	prev_was_vowel = word[0] in VOWELS
	for char in word[1:]:
		is_vowel = char in VOWELS
		if prev_was_vowel and not is_vowel:
			result += 1
		prev_was_vowel = is_vowel

	if (len(word) > 1 and word[0] in VOWELS
			and word.endswith('e') and not word[-2] in VOWELS):
		result += 1
	return result or 1


conjuction_en = r'and|but|or|yet|nor'
preposition_en = (
		'board|about|above|according to|across from'
		'|after|against|alongside|alongside of|along with'
		'|amid|among|apart from|around|aside from|at|away from'
		'|back of|because of|before|behind|below|beneath|beside'
		'|besides|between|beyond|but|by means of'
		'|concerning|considering|despite|down|down from|during'
		'|except|except for|excepting for|from among'
		'|from between|from under|in addition to|in behalf of'
		'|in front of|in place of|in regard to|inside of|inside'
		'|in spite of|instead of|into|like|near to|off'
		'|on account of|on behalf of|onto|on top of|on|opposite'
		'|out of|out|outside|outside of|over to|over|owing to'
		'|past|prior to|regarding|round about|round'
		'|since|subsequent to|together|with|throughout|through'
		'|till|toward|under|underneath|until|unto|up'
		'|up to|upon|with|within|without|across|along'
		'|by|of|in|to|near|of|from')
pronoun_en = (
		'i|me|we|us|you|he|him|she|her|it|they'
		'|them|thou|thee|ye|myself|yourself|himself'
		'|herself|itself|ourselves|yourselves|themselves'
		'|oneself|my|mine|his|hers|yours|ours|theirs|its'
		'|our|that|their|these|this|those|your')
words_en = collections.OrderedDict([
	('tobeverb', re.compile(
		r'\b(be|being|was|were|been|are|is)\b', re.IGNORECASE)),
	('auxverb', re.compile(
		r"\b(will|shall|cannot|may|need to|would|should"
		r"|could|might|must|ought|ought to|can't|can)\b", re.IGNORECASE)),
	('conjunction', re.compile(
		'\\b(%s)\\b' % conjuction_en, re.IGNORECASE)),
	('pronoun', re.compile(
		'\\b(%s)\\b' % pronoun_en, re.IGNORECASE)),
	('preposition', re.compile(
		'\\b(%s)\\b' % preposition_en, re.IGNORECASE)),
    # a bit limited, but this is exactly what the original style(1) did:
	('nominalization', re.compile(
		r'\b\w{3,}(tion|ment|ence|ance)\b', re.IGNORECASE | re.UNICODE)),
	])

beginnings_en = collections.OrderedDict([
	('pronoun', re.compile(
		'(^|\\n)(%s)\\b' % pronoun_en, re.IGNORECASE)),
	('interrogative', re.compile(
		r'(^|\n)(why|who|what|whom|when|where|how)\b', re.IGNORECASE)),
	('article', re.compile(
		r'(^|\n)(the|a|an)\b', re.IGNORECASE)),
	('subordination', re.compile(
		r"(^|\n)(after|because|lest|till|'til|although"
		r"|before|now that|unless|as|even if|provided that|provided"
		r"|until|as if|even though|since|as long as|so that"
		r"|whenever|as much as|if|than|as soon as|inasmuch"
		r"|in order that|though|while)\b", re.IGNORECASE)),
	('conjunction', re.compile(
		'(^|\\n)(%s)\\b' % conjuction_en, re.IGNORECASE)),
	('preposition', re.compile(
		'(^|\\n)(%s)\\b' % preposition_en, re.IGNORECASE)),
	])

conjuction_nl = 'en|maar|of|want|dus|noch'
preposition_nl = (
		"à|aan|ad|achter|behalve|beneden|betreffende|bij"
		"|binnen|blijkens|boven|buiten|circa|conform|contra"
		"|cum|dankzij|door|gedurende|gezien|hangende|in"
		"|ingevolge|inzake|jegens|krachtens|langs|met|middels"
		"|mits|na|naar|naast|nabij|namens|niettegenstaande"
		"|nopens|om|omstreeks|omtrent|ondanks|onder|ongeacht"
		"|onverminderd|op|over|overeenkomstig|per|plus|richting"
		"|qua|rond|rondom|sedert|staande|te|tegen|tegenover"
		"|ten|ter|tijdens|tot|tussen|uit|uitgezonderd|van"
		"|vanaf|vanuit|vanwege|versus|via|volgens|voor"
		"|voorbij|wegens|zonder")
pronoun_nl = (
		# persoonlijk voornaamwoord
		"ik|jij|je|u|hij|hem|zij|ze|haar|het"
		"|wij|we|ons|jullie|hen|hun"
		# wederkerend voornaamwoord
		"mij|me|mijzelf|mezelf|je|jezelf|uzelf"
		"|zich|zichzelf|haarzelf|onszelf"
		"|elkaar|elkaars|elkander|elkanders|mekaar|mekaars"
		# pers. vnw: archaisch
		"gij|ge"
		"|mijnen|deinen|zijnen|haren|onzen|uwen|hunnen|haren"
		"|mijner|deiner|zijner|harer|onzer|uwer|hunner|harer"
		"|mijnes|deines|zijnes|hares|onzes|uwes|hunnes|hares")
words_nl = collections.OrderedDict([
	('tobeverb', re.compile(
		r'\b(ben|bent|is|zijn|was|waren)\b', re.IGNORECASE)),
	('auxverb', re.compile(
		"\\b("
		# NB: past perfect forms of these verbs
		# ('gehad', 'geweest', 'geworden') are not auxiliary.
		# with past perfect verb
		"heb|hebt|heeft|hebben|had|hadden"
		"|word|wordt|worden|werd|werden"
		# "|ben|bent|is|zijn|was|waren"
		# with infinitive
		"|zal|zult|zullen|zou|zouden"
		"|kan|kan|kunt|kunnen|kon|konden"
		"|wil|wilt|willen|wilde|wilden|wou|wouden"
		"|moet|moeten|moest|moesten"
		# "|mag|mogen|mocht|mochten"
		# "|hoef|hoeft|hoeven|hoefde|hoefden"
		# "|doe|doet|doen|deed|deden"
		")\\b", re.IGNORECASE)),
	('conjunction', re.compile(
		'\\b(%s)\\b' % conjuction_nl, re.IGNORECASE)),
	('pronoun', re.compile(
		'\\b(%s)\\b' % pronoun_nl, re.IGNORECASE)),
	('preposition', re.compile(
		'\\b(%s)\\b' % preposition_nl, re.IGNORECASE)),
    # a bit limited, but this is exactly what the original style(1) did:
	('nominalization', re.compile(
		r'\b.{3,}(tie|heid|ing|end|ende)\b', re.IGNORECASE)),
	])
beginnings_nl = collections.OrderedDict([
	('pronoun', re.compile(
		'(^|\\n)(%s)\\b' % pronoun_nl, re.IGNORECASE)),
	('interrogative', re.compile(
		r'(^|\n)(wie|wat|waar|waarom|wanneer|hoe|welk|welke)\b',
		re.IGNORECASE)),
	('article', re.compile(
		r"(^|\n)(de|het|een|'t)\b", re.IGNORECASE)),
	('subordination', re.compile(
		"(^|\\n)("
		# onderschikkende voegwoorden
		"aangezien|als|alsof|behalve|daar|daarom|dat"
		"|derhalve|doch|doordat|hoewel|indien|mits|nadat"
		"|noch|ofschoon|omdat|ondanks|opdat|sedert|sinds"
		"|tenzij|terwijl|toen|totdat|voordat|wanneer"
		"|zoals|zodat|zodra|zonder dat"
		# infitief constructies
		"|om te)\\b", re.IGNORECASE)),
	('conjunction', re.compile(
		'(^|\\n)(%s)\\b' % conjuction_nl, re.IGNORECASE)),
	('preposition', re.compile(
		'(^|\\n)(%s)\\b' % preposition_nl, re.IGNORECASE)),
	])

conjuction_de = ('und|oder|aber|sondern|doch|nur|bloß|denn'
		'weder|noch|sowie')
preposition_de = (
	'aus|außer|bei|mit|nach|seit|von|zu'
	'|bis|durch|für|gegen|ohne|um|an|auf'
	'|hinter|in|neben|über|unter|vor|zwischen'
	'|anstatt|statt|trotz|während|wegen')
pronoun_de = (
	'ich|du|er|sie|es|wir|ihr'  # sie           # Nominativ
	'|mich|dich|ihn|uns|euch'  # sie             # Akkusativ
	'|mir|dir|ihm|ihnen'  # uns euch ihr         # Dativ
	'|mein|dein|sein|unser|euer'  # ihr          # Genitiv
	'|meiner|deiner|seiner|unserer|eurer|ihrer'  # Genitiv
	'|meine|deine|seine|unsere|eure|ihre'        # Genitiv
	'|meines|deines|seines|unseres|eures|ihres'  # Genitiv
	'|meinem|deinem|seinem|unserem|eurem|ihrem'  # Genitiv
	'|meinen|deinen|seinen|unseren|euren|ihren'  # Genitiv
		)
words_de = collections.OrderedDict([
	('tobeverb', re.compile("\\b("
		"sein|bin|bist|ist|sind|seid|war|warst|wart"
		"|waren|gewesen|wäre|wärst|wär|wären|wärt|wäret"
		")\\b", re.IGNORECASE)),
	('auxverb', re.compile("\\b("
		"haben|habe|hast|hat|habt|gehabt|hätte|hättest"
		"|hätten|hättet"
		"|werden|werde|wirst|wird|werdet|geworden|würde"
		"|würdest|würden|würdet"
		"|können|kann|kannst|könnt|konnte|konntest|konnten"
		"|konntet|gekonnt|könnte|könntest|könnten|könntet"
		"|müssen|muss|muß|musst|müsst|musste|musstest|mussten"
		"|gemusst|müsste|müsstest|müssten|müsstet"
		"|sollen|soll|sollst|sollt|sollte|solltest|solltet"
		"|sollten|gesollt"
		")\\b", re.IGNORECASE)),
	('conjunction', re.compile(
		'\\b(%s)\\b' % conjuction_de, re.IGNORECASE)),
	('pronoun', re.compile(
		'\\b(%s)\\b' % pronoun_de, re.IGNORECASE)),
	('preposition', re.compile(
		'\\b(%s)\\b' % preposition_de, re.IGNORECASE)),
	('nominalization', re.compile(
		r'\b.{3,}(ung|heit|keit|nis|tum)\b', re.IGNORECASE)),
	])
beginnings_de = collections.OrderedDict([
	('pronoun', re.compile(
		'(^|\\n)(%s)\\b' % pronoun_de, re.IGNORECASE)),
	('interrogative', re.compile(
		r'(^|\n)(wer|was|wem|wen|wessen|wo|wie|warum|weshalb|wann'
		r'|wieso|weswegen)\b', re.IGNORECASE)),
	('article', re.compile(
		r"(^|\n)(der|die|das|des|dem|den|ein|eine|einer|eines|einem|einen)\b",
		re.IGNORECASE)),
	('subordination', re.compile("(^|\\n)("
		# bei Nebensätzen
		"als|als dass|als daß|als ob|anstatt dass|anstatt daß"
		"|ausser dass|ausser daß|ausser wenn|bevor|bis|da|damit"
		"|dass|daß|ehe|falls|indem|je|nachdem|ob|obgleich"
		"|obschon|obwohl|ohne dass|ohne daß|seit|so daß|sodass"
		"|sobald|sofern|solange|so oft|statt dass|statt daß"
		"|während|weil|wenn|wenn auch|wenngleich|wie|wie wenn"
		"|wiewohl|wobei|wohingegen|zumal"
		# bei Infinitivgruppen
		"|als zu|anstatt zu|ausser zu|ohne zu|statt zu|um zu"
		")\\b", re.IGNORECASE)),
	('conjunction', re.compile(
		'(^|\\n)(%s)\\b' % conjuction_de, re.IGNORECASE)),
	('preposition', re.compile(
		'(^|\\n)(%s)\\b' % preposition_de, re.IGNORECASE)),
	])

# Long Dale-Chall word list of 3000 words recognized by 80 % of fifth graders
basicwords_en = frozenset("""
n't 'm 'll 'd 's 've
a able aboard about above absent accept accident account
ache aching acorn acre across act acts add address admire adventure afar afraid
after afternoon afterward afterwards again against age aged ago agree ah ahead
aid aim air airfield airplane airport airship airy alarm alike alive all alley
alligator allow almost alone along aloud already also always am America
American among amount an and angel anger angry animal another answer ant any
anybody anyhow anyone anything anyway anywhere apart apartment ape apiece
appear apple April apron are aren't arise arithmetic arm armful army arose
around arrange arrive arrived arrow art artist as ash ashes aside ask asleep at
ate attack attend attention August aunt author auto automobile autumn avenue
awake awaken away awful awfully awhile ax axe baa babe babies back background
backward backwards bacon bad badge badly bag bake baker bakery baking ball
balloon banana band bandage bang banjo bank banker bar barber bare barefoot
barely bark barn barrel base baseball basement basket bat batch bath bathe
bathing bathroom bathtub battle battleship bay be beach bead beam bean bear
beard beast beat beating beautiful beautify beauty became because become
becoming bed bedbug bedroom bedspread bedtime bee beech beef beefsteak beehive
been beer beet before beg began beggar begged begin beginning begun behave
behind being believe bell belong below belt bench bend beneath bent berries
berry beside besides best bet better between bib bible bicycle bid big bigger
bill billboard bin bind bird birth birthday biscuit bit bite biting bitter
black blackberry blackbird blackboard blackness blacksmith blame blank blanket
blast blaze bleed bless blessing blew blind blindfold blinds block blood bloom
blossom blot blow blue blueberry bluebird blush board boast boat bob bobwhite
bodies body boil boiler bold bone bonnet boo book bookcase bookkeeper boom boot
born borrow boss both bother bottle bottom bought bounce bow bowl bow-wow box
boxcar boxer boxes boy boyhood bracelet brain brake bran branch brass brave
bread break breakfast breast breath breathe breeze brick bride bridge bright
brightness bring broad broadcast broke broken brook broom brother brought brown
brush bubble bucket buckle bud buffalo bug buggy build building built bulb bull
bullet bum bumblebee bump bun bunch bundle bunny burn burst bury bus bush
bushel business busy but butcher butt butter buttercup butterfly buttermilk
butterscotch button buttonhole buy buzz by bye cab cabbage cabin cabinet cackle
cage cake calendar calf call caller calling came camel camp campfire can canal
canary candle candlestick candy cane cannon cannot canoe can't canyon cap cape
capital captain car card cardboard care careful careless carelessness carload
carpenter carpet carriage carrot carry cart carve case cash cashier castle cat
catbird catch catcher caterpillar catfish catsup cattle caught cause cave
ceiling cell cellar cent center cereal certain certainly chain chair chalk
champion chance change chap charge charm chart chase chatter cheap cheat check
checkers cheek cheer cheese cherry chest chew chick chicken chief child
childhood children chill chilly chimney chin china chip chipmunk chocolate
choice choose chop chorus chose chosen christen Christmas church churn
cigarette circle circus citizen city clang clap class classmate classroom claw
clay clean cleaner clear clerk clever click cliff climb clip cloak clock close
closet cloth clothes clothing cloud cloudy clover clown club cluck clump coach
coal coast coat cob cobbler cocoa coconut cocoon cod codfish coffee coffeepot
coin cold collar college color colored colt column comb come comfort comic
coming company compare conductor cone connect coo cook cooked cooking cookie
cookies cool cooler coop copper copy cord cork corn corner correct cost cot
cottage cotton couch cough could couldn't count counter country county course
court cousin cover cow coward cowardly cowboy cozy crab crack cracker cradle
cramps cranberry crank cranky crash crawl crazy cream creamy creek creep crept
cried croak crook crooked crop cross crossing cross-eyed crow crowd crowded
crown cruel crumb crumble crush crust cry cries cub cuff cup cuff cup cupboard
cupful cure curl curly curtain curve cushion custard customer cut cute cutting
dab dad daddy daily dairy daisy dam damage dame damp dance dancer dancing dandy
danger dangerous dare dark darkness darling darn dart dash date daughter dawn
day daybreak daytime dead deaf deal dear death December decide deck deed deep
deer defeat defend defense delight den dentist depend deposit describe desert
deserve desire desk destroy devil dew diamond did didn't die died dies
difference different dig dim dime dine ding-dong dinner dip direct direction
dirt dirty discover dish dislike dismiss ditch dive diver divide do dock doctor
does doesn't dog doll dollar dolly done donkey don't door doorbell doorknob
doorstep dope dot double dough dove down downstairs downtown dozen drag drain
drank draw drawer draw drawing dream dress dresser dressmaker drew dried drift
drill drink drip drive driven driver drop drove drown drowsy drub drum drunk
dry duck due dug dull dumb dump during dust dusty duty dwarf dwell dwelt dying
each eager eagle ear early earn earth east eastern easy eat eaten edge egg eh
eight eighteen eighth eighty either elbow elder eldest electric electricity
elephant eleven elf elm else elsewhere empty end ending enemy engine engineer
English enjoy enough enter envelope equal erase eraser errand escape eve even
evening ever every everybody everyday everyone everything everywhere evil exact
except exchange excited exciting excuse exit expect explain extra eye eyebrow
fable face facing fact factory fail faint fair fairy faith fake fall false
family fan fancy far faraway fare farmer farm farming far-off farther fashion
fast fasten fat father fault favor favorite fear feast feather February fed
feed feel feet fell fellow felt fence fever few fib fiddle field fife fifteen
fifth fifty fig fight figure file fill film finally find fine finger finish
fire firearm firecracker fireplace fireworks firing first fish fisherman fist
fit fits five fix flag flake flame flap flash flashlight flat flea flesh flew
flies flight flip flip-flop float flock flood floor flop flour flow flower
flowery flutter fly foam fog foggy fold folks follow following fond food fool
foolish foot football footprint for forehead forest forget forgive forgot
forgotten fork form fort forth fortune forty forward fought found fountain four
fourteen fourth fox frame free freedom freeze freight French fresh fret Friday
fried friend friendly friendship frighten frog from front frost frown froze
fruit fry fudge fuel full fully fun funny fur furniture further fuzzy gain
gallon gallop game gang garage garbage garden gas gasoline gate gather gave gay
gear geese general gentle gentleman gentlemen geography get getting giant gift
gingerbread girl give given giving glad gladly glance glass glasses gleam glide
glory glove glow glue go going goes goal goat gobble God god godmother gold
golden goldfish golf gone good goods goodbye good-by goodbye good-bye
good-looking goodness goody goose gooseberry got govern government gown grab
gracious grade grain grand grandchild grandchildren granddaughter grandfather
grandma grandmother grandpa grandson grandstand grape grapes grapefruit grass
grasshopper grateful grave gravel graveyard gravy gray graze grease great green
greet grew grind groan grocery ground group grove grow guard guess guest guide
gulf gum gun gunpowder guy ha habit had hadn't hail hair haircut hairpin half
hall halt ham hammer hand handful handkerchief handle handwriting hang happen
happily happiness happy harbor hard hardly hardship hardware hare hark harm
harness harp harvest has hasn't haste hasten hasty hat hatch hatchet hate haul
have haven't having hawk hay hayfield haystack he head headache heal health
healthy heap hear hearing heard heart heat heater heaven heavy he'd heel height
held hell he'll hello helmet help helper helpful hem hen henhouse her hers herd
here here's hero herself he's hey hickory hid hidden hide high highway hill
hillside hilltop hilly him himself hind hint hip hire his hiss history hit
hitch hive ho hoe hog hold holder hole holiday hollow holy home homely homesick
honest honey honeybee honeymoon honk honor hood hoof hook hoop hop hope hopeful
hopeless horn horse horseback horseshoe hose hospital host hot hotel hound hour
house housetop housewife housework how however howl hug huge hum humble hump
hundred hung hunger hungry hunk hunt hunter hurrah hurried hurry hurt husband
hush hut hymn I ice icy I'd idea ideal if ill I'll I'm important impossible
improve in inch inches income indeed Indian indoors ink inn insect inside
instant instead insult intend interested interesting into invite iron is island
isn't it its it's itself I've ivory ivy jacket jacks jail jam January jar jaw
jay jelly jellyfish jerk jig job jockey join joke joking jolly journey joy
joyful joyous judge jug juice juicy July jump June junior junk just keen keep
kept kettle key kick kid kill killed kind kindly kindness king kingdom kiss
kitchen kite kitten kitty knee kneel knew knife knit knives knob knock knot
know known lace lad ladder ladies lady laid lake lamb lame lamp land lane
language lantern lap lard large lash lass last late laugh laundry law lawn
lawyer lay lazy lead leader leaf leak lean leap learn learned least leather
leave leaving led left leg lemon lemonade lend length less lesson let let's
letter letting lettuce level liberty library lice lick lid lie life lift light
lightness lightning like likely liking lily limb lime limp line linen lion lip
list listen lit little live lives lively liver living lizard load loaf loan
loaves lock locomotive log lone lonely lonesome long look lookout loop loose
lord lose loser loss lost lot loud love lovely lover low luck lucky lumber lump
lunch lying ma machine machinery mad made magazine magic maid mail mailbox
mailman major make making male mama mamma man manager mane manger many map
maple marble march March mare mark market marriage married marry mask mast
master mat match matter mattress may May maybe mayor maypole me meadow meal
mean means meant measure meat medicine meet meeting melt member men mend meow
merry mess message met metal mew mice middle midnight might mighty mile milk
milkman mill miler million mind mine miner mint minute mirror mischief miss
Miss misspell mistake misty mitt mitten mix moment Monday money monkey month
moo moon moonlight moose mop more morning morrow moss most mostly mother motor
mount mountain mouse mouth move movie movies moving mow Mr. Mrs. much mud muddy
mug mule multiply murder music must my myself nail name nap napkin narrow nasty
naughty navy near nearby nearly neat neck necktie need needle needn't Negro
neighbor neighborhood neither nerve nest net never nevermore new news newspaper
next nibble nice nickel night nightgown nine nineteen ninety no nobody nod
noise noisy none noon nor north northern nose not note nothing notice November
now nowhere number nurse nut oak oar oatmeal oats obey ocean o'clock October
odd of off offer office officer often oh oil old old-fashioned on once one
onion only onward open or orange orchard order ore organ other otherwise ouch
ought our ours ourselves out outdoors outfit outlaw outline outside outward
oven over overalls overcoat overeat overhead overhear overnight overturn owe
owing owl own owner ox pa pace pack package pad page paid pail pain painful
paint painter painting pair pal palace pale pan pancake pane pansy pants papa
paper parade pardon parent park part partly partner party pass passenger past
paste pasture pat patch path patter pave pavement paw pay payment pea peas
peace peaceful peach peaches peak peanut pear pearl peck peek peel peep peg pen
pencil penny people pepper peppermint perfume perhaps person pet phone piano
pick pickle picnic picture pie piece pig pigeon piggy pile pill pillow pin pine
pineapple pink pint pipe pistol pit pitch pitcher pity place plain plan plane
plant plate platform platter play player playground playhouse playmate
plaything pleasant please pleasure plenty plow plug plum pocket pocketbook poem
point poison poke pole police policeman polish polite pond ponies pony pool
poor pop popcorn popped porch pork possible post postage postman pot potato
potatoes pound pour powder power powerful praise pray prayer prepare present
pretty price prick prince princess print prison prize promise proper protect
proud prove prune public puddle puff pull pump pumpkin punch punish pup pupil
puppy pure purple purse push puss pussy pussycat put putting puzzle quack quart
quarter queen queer question quick quickly quiet quilt quit quite rabbit race
rack radio radish rag rail railroad railway rain rainy rainbow raise raisin
rake ram ran ranch rang rap rapidly rat rate rather rattle raw ray reach read
reader reading ready real really reap rear reason rebuild receive recess record
red redbird redbreast refuse reindeer rejoice remain remember remind remove
rent repair repay repeat report rest return review reward rib ribbon rice rich
rid riddle ride rider riding right rim ring rip ripe rise rising river road
roadside roar roast rob robber robe robin rock rocky rocket rode roll roller
roof room rooster root rope rose rosebud rot rotten rough round route row
rowboat royal rub rubbed rubber rubbish rug rule ruler rumble run rung runner
running rush rust rusty rye sack sad saddle sadness safe safety said sail
sailboat sailor saint salad sale salt same sand sandy sandwich sang sank sap
sash sat satin satisfactory Saturday sausage savage save savings saw say scab
scales scare scarf school schoolboy schoolhouse schoolmaster schoolroom scorch
score scrap scrape scratch scream screen screw scrub sea seal seam search
season seat second secret see seeing seed seek seem seen seesaw select self
selfish sell send sense sent sentence separate September servant serve service
set setting settle settlement seven seventeen seventh seventy several sew shade
shadow shady shake shaker shaking shall shame shan't shape share sharp shave
she she'd she'll she's shear shears shed sheep sheet shelf shell shepherd shine
shining shiny ship shirt shock shoe shoemaker shone shook shoot shop shopping
shore short shot should shoulder shouldn't shout shovel show shower shut shy
sick sickness side sidewalk sideways sigh sight sign silence silent silk sill
silly silver simple sin since sing singer single sink sip sir sis sissy sister
sit sitting six sixteen sixth sixty size skate skater ski skin skip skirt sky
slam slap slate slave sled sleep sleepy sleeve sleigh slept slice slid slide
sling slip slipped slipper slippery slit slow slowly sly smack small smart
smell smile smoke smooth snail snake snap snapping sneeze snow snowy snowball
snowflake snuff snug so soak soap sob socks sod soda sofa soft soil sold
soldier sole some somebody somehow someone something sometime sometimes
somewhere son song soon sore sorrow sorry sort soul sound soup sour south
southern space spade spank sparrow speak speaker spear speech speed spell
spelling spend spent spider spike spill spin spinach spirit spit splash spoil
spoke spook spoon sport spot spread spring springtime sprinkle square squash
squeak squeeze squirrel stable stack stage stair stall stamp stand star stare
start starve state station stay steak steal steam steamboat steamer steel steep
steeple steer stem step stepping stick sticky stiff still stillness sting stir
stitch stock stocking stole stone stood stool stoop stop stopped stopping store
stork stories storm stormy story stove straight strange stranger strap straw
strawberry stream street stretch string strip stripes strong stuck study stuff
stump stung subject such suck sudden suffer sugar suit sum summer sun Sunday
sunflower sung sunk sunlight sunny sunrise sunset sunshine supper suppose sure
surely surface surprise swallow swam swamp swan swat swear sweat sweater sweep
sweet sweetness sweetheart swell swept swift swim swimming swing switch sword
swore table tablecloth tablespoon tablet tack tag tail tailor take taken taking
tale talk talker tall tame tan tank tap tape tar tardy task taste taught tax
tea teach teacher team tear tease teaspoon teeth telephone tell temper ten
tennis tent term terrible test than thank thanks thankful Thanksgiving that
that's the theater thee their them then there these they they'd they'll they're
they've thick thief thimble thin thing think third thirsty thirteen thirty this
thorn those though thought thousand thread three threw throat throne through
throw thrown thumb thunder Thursday thy tick ticket tickle tie tiger tight till
time tin tinkle tiny tip tiptoe tire tired title to toad toadstool toast
tobacco today toe together toilet told tomato tomorrow ton tone tongue tonight
too took tool toot tooth toothbrush toothpick top tore torn toss touch tow
toward towards towel tower town toy trace track trade train tramp trap tray
treasure treat tree trick tricycle tried trim trip trolley trouble truck true
truly trunk trust truth try tub Tuesday tug tulip tumble tune tunnel turkey
turn turtle twelve twenty twice twig twin two ugly umbrella uncle under
understand underwear undress unfair unfinished unfold unfriendly unhappy unhurt
uniform United States unkind unknown unless unpleasant until unwilling up upon
upper upset upside upstairs uptown upward us use used usefulç valentine valley
valuable value vase vegetable velvet very vessel victory view village vine
violet visit visitor voice vote wag wagon waist wait wake waken walk wall
walnut want war warm warn was wash washer washtub wasn't waste watch watchman
water watermelon waterproof wave wax way wayside we weak weakness weaken wealth
weapon wear weary weather weave web we'd wedding Wednesday wee weed week we'll
weep weigh welcome well went were we're west western wet we've whale what
what's wheat wheel when whenever where which while whip whipped whirl whisky
whiskey whisper whistle white who who'd whole who'll whom who's whose why
wicked wide wife wiggle wild wildcat will willing willow win wind windy
windmill window wine wing wink winner winter wipe wire wise wish wit witch with
without woke wolf woman women won wonder wonderful won't wood wooden woodpecker
woods wool woolen word wore work worker workman world worm worn worry worse
worst worth would wouldn't wound wove wrap wrapped wreck wren wring write
writing written wrong wrote wrung yard yarn year yell yellow yes yesterday yet
yolk yonder you you'd you'll young youngster your yours you're yourself
yourselves youth you've
""".lower().split())

# 3000 most frequent word tokens in Sonar 500 corpus
basicwords_nl = frozenset("""
. de , van het een en in dat is op te zijn voor met ik die niet ) ( : " maar er
' aan - ook je als om ? hij ze bij dan nog was naar uit of door we heeft over
wat al tot worden meer hebben wordt geen wel jaar kan ! dit nu zich zo hun deze
werd moet mijn haar na kunnen zou veel tegen ... 1 had ; twee heb zal daar toch
andere goed eerste wil moeten waar mensen / onder nieuwe gaat gaan 2 dus hem u
ons me ben weer hier alleen toen omdat hoe doen zegt onze alle heel tussen
maken mij grote eens '' zij waren uur komt iets komen staat zoals euro wij 3
drie af * volgens tijd want altijd één zelf wie zonder mee weet man vooral
eigen ja tweede enkele leven zelfs plaats keer zien toe dag alles jaren laten
the 4 men echt werden zullen willen nooit weg 0 % zeker kinderen laatste kon
tijdens net week kwam terug mag .. 5 land krijgen europese staan per werk
zeggen binnen minder even procent aantal steeds laat niets blijft 6 miljoen
beter samen ging blijven werken zo'n elkaar zit verder anders rond nederland
ten hadden gewoon lang artikel vier misschien via geven vandaag zei iedereen \]
eerst moest jij re goede waarom 10 sinds pas geleden weten later hele geld
wereld zouden mogelijk hen iemand vraag brussel groot belgië vrouw doet houden
deel bijna elke vinden echter huis gisteren vaak \[ vanaf nodig maakt kreeg
opnieuw vind allemaal zitten snel vlaamse nemen ter volgende vijf krijgt achter
der maanden dagen minister stad erg uw denk natuurlijk da la politie nee derde
belgische nederlandse weinig amerikaanse terwijl ligt ga zeer a helemaal
geweest vorig hebt den 7 duidelijk kleine 20 commissie regering vrouwen daarom
gemaakt wanneer manier jullie beste spelen enige vindt 8 waarin naam soms
eigenlijk verschillende groep zie begin paar tien lijkt bijvoorbeeld eerder
gedaan graag weken onderzoek gezien geval mogen bent 15 europa oude voorzitter
partij vragen geeft genoeg seizoen enkel landen open vorige bovendien frank
meteen vlaanderen wilde moment welke kans zes politieke antwerpen zondag zaken
club stond raad bedrijf mannen elk 12 brengen buiten ooit beetje grootste hand
naast slechts 9 ziet probleem bekend nieuw titel niemand problemen vader maakte
school 25 zaterdag vrij ander markt dood 11 gent thuis morgen dezelfde moeilijk
denken aldus jonge einde daarna le 't bestaat moeder stellen eind vast jan
blijkt boven kijken zetten water boek vanuit muziek 30 zaak kind ouders deed
zowel 14 wedstrijd ` 18 vond bedrijven ploeg zoveel gemeente meeste afgelopen
soort punten meest kunt maand les quote geworden & extra zodat sociale meter
film gebruik gebruikt doe beide kom liggen franse halen vroeg zag gezegd
president politiek dingen uiteindelijk langs = hoop valt houdt september b
sommige miljard lange best d belang des vlaams familie zeg liet zat hoofd | ge
13 bezig à helft nationale neemt juni 16 zorgen ervan sterk jongeren auto et
mei stuk daarmee handen januari niks lid kwamen konden druk trouwens ongeveer
start immers frankrijk 17 internationale nieuws recht vele p belangrijk prijs
2004 oktober bank rol overheid waarop informatie foto burgemeester waarbij ogen
langer vroeger betalen mooi parlement brugge juist echte kant verhaal bepaalde
zichzelf beginnen waarvan volledig verordening inderdaad basis nadat klein hoge
juli zin lopen duitse wist mens zoon economische anderen hetzelfde radio oorlog
werkt toekomst speelt 2005 's gaf horen meestal huidige reden periode stelt
minuten .... acht verwacht totaal gelijk bestaan daarbij 19 vol december succes
god maart 21 daarvoor hoor gehad ruim ni amsterdam begon loopt idee april
begint licht > eten precies kosten bleef slecht hoog duitsland vorm betekent
eeuw financiële vallen rest ver vrijdag zeven woord jou peter kort grond
premier programma bleek wachten genomen 24 spelers 50 22 gegeven terecht 100
gebied ronde verenigde belangrijke zware partijen plaatsen z'n nr. aandacht
november nam waardoor punt rode gelukkig rust dienst nummer plan staten woorden
kijk publiek hoewel rekening dacht gemeenschap situatie zoek unie volgend
voorbije mocht feit zoeken gebeurt intussen new weekend kamer 23 moesten klaar
koning leden doel trekken gebruiken zwaar verkiezingen britse helpen leren + i
belangrijkste vs trainer centrum gekomen gevonden kregen augustus vierde kennen
waarschijnlijk namen dollar mooie volgen dochter «ik waarmee 2003 daarvan
vertelt buitenlandse lijst komende spreken 2000 gevolg tegenover korte proberen
half vrije maandag top won zet gevallen gingen onderwijs straat vlak steun and
winnen daardoor beeld vrienden maatregelen denkt ene bijzonder nederlands
ruimte vormen leggen europees liep geschiedenis finale gevoel inwoners ondanks
volgt niveau 2001 februari praten hard geloof richting strijd leuk buurt cd&v
blz. heer paul gratis antwoord betrokken voorlopig  gekregen 2002 wellicht
verloren volk begonnen directeur gebracht vld lol economie speelde project --
schepen info winst organisatie voetbal werknemers stemmen 40 verschil momenteel
gebeuren brengt wet systeem internet hoger kilometer leuven nacht oud tom alsof
marc viel liever kopen actie woensdag kun betreft ieder sport wonen gehouden
bevolking kerk wegens fc akkoord vraagt straks cijfers lezen orde algemeen
leiden beleid bart normaal uiteraard ma twintig rijden kost hart jongens
producten wagen oog richtlijn l én irak 28 meisje samenwerking blijkbaar kent
hulp erop klanten jongen namelijk macht 26 hoogte nie 27 schrijven graden
genoemd nauwelijks anderlecht verleden boeken waarde heen oplossing 2006
ziekenhuis du eén luc reeks vriend provincie lokale contact controle donderdag
vanavond witte overigens overal avond geboren deur kiezen ondertussen besluit
keuze hield voldoende blij bepaald media sterke zomer zwarte zee geef rechter
liefde lijn bestuur nou team taal beroep tekst resultaat regio to wou
resultaten oostende dinsdag krant termijn voorbeeld indien slag vertrouwen
doelpunten stand kennis s voorstel inmiddels hoeveel wijze 31 geweld vergeten
david voorbij vervolgens dieren mening gezet groter dankzij woning frans voeren
brand foto's bezoek «de m gesteld telkens 60 hé vertellen leeftijd ontwikkeling
ervaring cultuur geldt lag plannen krijg meisjes aandeel gevraagd km kunst
liefst nogal ervoor china leger lidstaten leerlingen haalde kracht bracht
sprake dicht johan dergelijke ne aandelen crisis los stap groen meerderheid
bezoekers kritiek gewonnen parijs actief eindelijk bedoeling halve persoon ken
tevreden legt iedere delen beslissing 29 relatie kop personen gezicht schreef
betere leek israël leider behalve noemen fout mechelen slechte jong geheel
wacht john daarnaast verkocht gevolgen daarin trekt bang negen bieden c ergens
universiteit klasse voorkomen gang jongste maak gegevens lichaam vervangen
lekker opdracht ii stelde omgeving gebeurd neem minstens direct 2007 «we
slachtoffer groei rug lucht brusselse prijzen bekende verkeer personeel raakte
he gegaan gebouw patrick name kaart gij hou onlangs gewone pakken kwaliteit
twaalf wilt heet scholen moeite midden links doden juiste geschreven «het
verkoop kader amerika tafel voelen stem website bal york voorzien regels sector
museum gesloten advies slachtoffers hoeft biedt contract kansen verkopen jouw
openbare k last plus italiaanse band spel wedstrijden rechtbank verband alvast
indruk hoort activiteiten genk federale 80 gezin antwerpse hoogste un italië
bouwen productie politici voorstellen proces mogelijke beslist spanje _ neer
invloed dertig middelen verslag spreekt gespeeld dirk word opgenomen pb
vastgesteld gebeurde omzet geloven zette 70 leiding zoiets klinkt justitie
stonden telt on amper wilden inzake discussie zon for optreden algemene
gemeenten o feiten peeters ministerie veld dans e moord ontstaan gevaar stoppen
speciale match duur dragen belgen veiligheid aanwezig centrale beperkt trok t
daarop 35 tegenwoordig onmiddellijk loop bewoners rusland hoofdstad verliezen
tv veranderen verlies baan keren kwestie buitenland honderd schade vuur rustig
tel. bed londen 00 brief café eenmaal fr. broer diverse jeugd kortrijk voel
bush vaste gesprek bijlage 1999 merk openbaar mevrouw enorm bron maatschappij
samenleving beelden wim werkte spaanse dank vanwege bedrag sp.a voelt reactie
gehoord dient steden ok beschikking pijn aarde wa toepassing stel schrijft 200
gekozen collega's burgers allerlei dorp hangt post erin provinciale positie cel
internationaal eisen geel haalt 90 advocaat koers 1996 enorme persoonlijke
verlaten kwijt zolang absoluut banken beurs gebrek vriendin goeie kabinet
stappen zege nochtans energie verloor sluiten functie binnenkort positief
standard vrijheid afstand plek winkel welk zorgt voormalige volle verdrag
leterme sociaal hans tegelijk studenten dl rotterdam geluk zowat jammer
roeselare mogelijkheden amerikanen moderne tonen bestaande reed noemt televisie
makkelijk debat mis lijken haag gemiddelde carrière site officiële aanleiding
veroordeeld diensten 1995 bereiken reeds eu sneller regelmatig plots eveneens
geraakt westen wk zorg hof technische das leidt grotere initiatief militaire
auto's betaald terrein hopen russische vijftien grens ontvangen woont pour
sommigen rechten omstandigheden bureau voelde groepen vakantie bereid groene
gewerkt val sfeer respect vlamingen bewijzen oh mogelijkheid gouden probeert
verwachten brandweer rij volledige gemiddeld vijfde hotel rekenen bestuurder
overwegende bedoeld geert wint regen perfect verantwoordelijk persoonlijk blijf
voordeel vijftig product moslims dienen wind n bus comité totale opleiding
redenen gesproken une kijkt risico hasselt voet deden dagelijks waarvoor
beweging nadien kwaad dak bereikt voorsprong vonden gericht klassieke officieel
turkije michel gemakkelijk \\ zaal hangen zaten 1998 werking drinken
overwinning michael films mezelf kaarten handel stil rijk noch droog zomaar
voorwaarden meneer schuld se verhalen vakbonden natuur aangezien verplicht
overeenkomst bedraagt steken rapport mond nodige maakten helaas prins zuiden
dossier uren diep erbij eerlijk studie as pak eigenaar slaan hogere klopt wegen
genieten bergen tour charleroi betreffende bloed uitgevoerd
verantwoordelijkheid jaarlijks thomas kampioen overleden sprak 2008 rechts
vaker leveren coach sturen job overleg station keek you allen parket waaronder
centraal deelnemers gestolen pieter reis nederlanders uitspraak alweer waard
boodschap zwart verklaring engeland speler duizenden toegang tijden eg publieke
gedacht opgepakt h f verboden geregeld bewust lacht angst eraan geplaatst sorry
park que elf bouw zulke cd begrijpen gek ware gele bedankt vereniging voordat
probeerde schrijver gevangenis islam koop hoopt positieve huizen laag waregem
competitie kleur guy wijn versie slapen verdwenen karakter honderden overtuigd
ah harde speciaal onmogelijk bos praktijk 45 albert zoekt waarheid zagen
belgisch zogenaamde di gehaald tientallen noorden 1994 gisteravond koen
afdeling boete lager gewond hoorde chinese rood boom aanpak 75 daarover dokter
nergens beneden directie gevolgd woningen 32 ongeval wees tuin gedurende ding
merken brussels stukken gebouwd rechtstreeks doelman degelijk gentse zekerheid
gedrag democratie toont vooruit 500 x veilig huwelijk passen mijnheer
tentoonstelling aalst lachen wakker wijzen warm baas soorten hond leeft m'n
gewijzigd daders karel vóór ermee onderzoeken qui bert wallonië bezit campagne
klant verdwijnen keuken doordat velen tijdje partner steven welkom ziekte
bijzondere principe raken robert sk effect lichte concurrentie besloten
reageren grenzen groetjes neen medewerkers materiaal ontslag engels turkse
levert model gebieden ofwel lage afgesloten poging ouder hoek serieus
historische voortaan bekijken vreemd vergelijking volgde kern starten veranderd
onderhandelingen herman draait probeer muur jezelf restaurant agenten ogenblik
noodzakelijk fiets gebouwen stijl au vrede aanvankelijk ziek taak vlucht kust
aanbod kim verkeerd engelse duizend verdienen beurt vroegere vertelde 33 eiland
kevin wijst sta begrip past roman onderwerp fortis kandidaten feest flink legde
mate collega soldaten betrekking voeten ondernemingen opgericht moeilijke
oorzaak wijk vlees hè 'n bescherming geest straf woordvoerder japan trein
meerdere verschillen tenminste est traditionele draaien behandeling 1997 zover
combinatie organiseren smaak ie beiden wereldoorlog computer milieu forum vormt
wél gelegen vertrekken recente type limburg oudere wit lief plezier voortdurend
ton manager zestig voort spoor lieten eur verbonden polen avonds bef projecten
tevens j. zorgde inhoud elders reacties il gemeenteraad kende tachtig janssens
uitgebreid vrijwel winnaar bewijs lees vzw veertig neus middel opvallend banen
eeg baby iran instellingen gelegenheid onderneming leuke gerecht vanmorgen
dikke generatie yves festival begrijp werkelijkheid toestand afrika bedoelde
luik nationaal betaalt selectie financieel vielen el eric winter oudste
economisch zelden afkomstig besloot evenwel commerciële reageert bevat c8 blok
sluit franstalige spijt thuisploeg bedoel aanval slot ploegen kv duren mark
dieven leidde vertrek ministers vandaar dichter aanslag organiseert verdere
spits begroting sterker luisteren ideeën verhofstadt podium klacht veroorzaakt
maria waalse plaatselijke acties n-va doelpunt israëlische willem blijken
behoorlijk beschikbaar georganiseerd telefoon mochten wapens par gebaseerd live
twijfel 65 eruit middag bleven 150 olympische operatie 300 oppervlakte
organisaties tegenstelling it anderzijds duurt goe beschouwd drugs deuren pers
oppositie afscheid binnenlandse thema 34 jezus definitief bepalingen serie
louis snelheid relatief bepalen verkeerde blik ongetwijfeld winkels culturele
papier joden afhankelijk olie minuut snelle kandidaat utrecht stof eer
weliswaar eventueel ernstig 36 interessant oosten mama lot 99 hoeven groeien
achteraf gelezen international bod plaatse afghanistan vergadering redactie
prima vervoer procedure bellen helpt allez zo. b. 1992 praat 1993 sloeg belg
zodra redden m. oké min onderdeel uitleg netwerk scoorde lukt willy ontdekt
hoofdpunten trots tenslotte lokeren groot-brittannië uitbreiding droom p. arm
fase clubs erger training filip komst tc gesprekken 46 eentje specifieke
medische oprichting set behoort aanslagen houding armen missen topic studio
betekenis jaarlijkse vliegen behouden mooiste getroffen berichten waarden
stijging militairen programma's bijdrage stijn financiën vb visie
belangstelling glas hopelijk seconden verzoek gerust sindsdien koningin vis
eenvoudig haal vermijden andré meent standaard gelegd dreigt talent destijds
bomen koffie da's gekocht ps draagt bestond 1991 gevaarlijk philippe d.
commentaar roepen 1990 patiënten vliegtuig verjaardag gasten aanwezigheid zak
chris <p> fouten ineens fijn gezondheid theater zult 02 inzet instituut
palestijnse teken werkelijk brug wisten gezond troepen uitvoering luchthaven
gemeenschappen auteur reizen dames rock george verzet vrees democratische
kranten gezinnen bericht kleuren vermoord koninklijke defensie benen menselijke
vreemde 55 turnhout joodse allebei redelijk a. editie kindje haast kiest stukje
pierre eddy jos gebleven partners jarenlang investeringen zus gelet datum
standpunt vermoedelijk waarna < motor sur voorstelling vergeet maxima
commissaris professor beschikken zeventig japanse minste martin erik vooraf
literatuur voorkeur kwart euh schip technisch roept voorgesteld show scoren
india sporen slagen tekort prachtige individuele 01 regionale gezocht inkomsten
voornamelijk 38 compleet negatieve nederlaag sven lagere beveren boonen wens
vermogen v. conflict goedgekeurd dubbele geslaagd 98 teksten hallo stichting
toevallig goud negentig pagina tegenstander antwoorden koninkrijk 37 traditie
roger mist behoren stijgen stadsbestuur letterlijk mekaar kleinere vrt westerse
natuurlijke interview straten toon gsm kris goederen zekere geleid belastingen
g beleggers vanmiddag haven rijdt scheidsrechter overeenkomstig streek speelden
wetenschap hiervoor klachten industrie zone uitzondering richten arbeiders
pakte zesde burger slaat inkomen ervaren ontwikkelen prachtig tegelijkertijd
eventuele verklaarde washington tijdelijk link rand zingen bel seks kleren
hiermee communautaire republiek agenda hugo onafhankelijk voert verdeeld wagens
digitale momenten leiders gemeentebestuur gedachten externe heilige vannacht
rome aangeboden gewezen lierse regeling waaraan an 42 hemel geheim islamitische
planten ontslagen noemde tja beschermen aangepast gestuurd instelling raakt
verscheen verdient concert oranje vrouwelijke spa kwartaal absolute feite stop
ruzie verschenen fans verschijnen buren 250 onvoldoende geschorst fietsen
duidelijke 97 voldoen idd qua interne jean ambtenaren been steunen wereldwijd
supporters beslissen gewest ek verbeteren verdachte renners steeg tienen
voorbereiding verandering zicht treden dromen madrid reclame interesse nood
dader sterven oostenrijk aanzien world conclusie normale omhoog sloot
lid-staten behoefte ontstond papa leverde management consument blauwe
uitsluitend omroep heren rit ingediend durven landbouw overname langzaam beker
simpel paus diens uiterlijk verteld bier evolutie begrepen brede verliest
walter kantoor beschikt nummers test kamp my league geleerd 400 jacques
afspraken maximaal gaven pensioen koos dringend aangehouden autoriteiten tal
ernstige achtergrond dr. luister wetgeving dure dier volop 44 allochtonen
hoeveelheid instantie es opgesteld daarentegen piet tenzij geluid behandeld
bewezen menen contacten steve verdediging favoriete y geraken maat boeren
ongeluk gelooft uitvoeren geopend stoffen afloop miljoenen schoenen c. riep
anderhalf moeilijker journalisten beslag lieve adres fonds maatschappelijke
gedeelte analyse meester klas tony wetenschappelijke regisseur boot katholieke
c6 markten dendermonde gemeenschappelijke gast vn mede san bezoeken arts out
gebruikte belangen afspraak nachts alsnog liepen investeren gelden categorie
hierbij getuigen elementen blad vlot bond ramp treedt trap ring normen teveel
prestatie tim bloemen verandert mol grotendeels voedsel bijdragen 85 opvolger
overige gevoelens gebeurtenissen zwanger uitspraken kunstenaar zulte eindigde
logisch aard verdedigen omwille berlijn communicatie waarover usd moe gestegen
ontwikkeld wensen g. strategie alternatief vogels vaststelling 39 stilaan
chauffeur geprobeerd zetels 43 slaap evenwicht raakten gezelschap geweldig
album dubbel lagen wezen huid erover kunstenaars voormalig leest opnemen banden
cercle fiscale aanleg dik deelnemen wolken verdacht united voertuig rijke u.
onbekende 1000 record kristof onderzocht opklaringen warme structuur ideale
ochtend veertien 48 bekeken stuur voorwaarde integratie sneeuw groeit tellen
openen vertrokken oorsprong aanmerking creëren studies be v pvda km² ajax gat
populair nederlander ingezet 1989 portugal belasting negatief uiterst grondwet
beheer moskou vloer roken artsen bevindt stevig prestaties lijden onderzoekers
stadion gedragen evenmin schuldig brood ach bijkomende formule tekenen verre
luidt iii voren budget s. voorbeelden overzicht hierdoor lastig techniek vos
vrachtwagen oudenaarde technologie duitsers enig gaten gelijke zwitserland
koppel berg ce trouw loon collectie werkgevers koud leeg eh guido opening
beperkte voorzichtig vermeld gepland fabriek rente richard toekomstige schoot
eet stopt bob aandeelhouders schrik controleren christelijke toeristen city
risico's 20.00 james kilo gevoerd breed geschikt vluchten oorspronkelijke
sint-niklaas kapitaal kanten greep werkloosheid maal oren minst 41 schorsing
rolleyes belangrijker smet inspanningen historisch pakt real getrokken
kruispunt vorst that zweden trek love st. las beperken verhouding verwachtingen
bestuurders liter toonde evenveel opzichte firma danny arabische verklaard
breken france westerlo achterstand hoi leert raar uitslag € afrikaanse vandaan
journalist klagen geslagen charles kwartier maes zakken schoon onafhankelijke
liedje gewicht realiteit socialistische werkgever meldt zen stevige stemming
bedragen terugkeer aa barcelona boos regel details kleiner tmf klap tis acteur
passagiers aardig spanning gedachte big verscheidene vechten finales schat
fransen concrete rik oproep goedkoper verenigd onzer slotte protest jo bruno
uitstekend integendeel schieten imago onderdelen theorie gedwongen franstaligen
locatie 20.30 overleed cda tegenstanders zevende zwakke concept martens
overtuigen bronnen constant populaire boord jeroen pink fantastisch ingang
linkse college amsterdamse belga religieuze opent onafhankelijkheid vieren
toerisme no haat politicus 2009 congo luxemburg meegemaakt iraakse golf aankoop
gooien kasteel tips coalitie mss betekenen revolutie overeenstemming redacteur
bibliotheek saddam verklaart maatregel champions marcel kiezers dode «in 52
brak vtm zaventem zijde overgenomen reageerde arme super hetgeen paard naartoe
juridische naties zestien opmerkelijk dagelijkse vroege geweten melk vvd stilte
hielden identiteit aangenomen 2010 getrouwd dertien leer pleit portefeuille dom
respectievelijk duo diezelfde makkelijker stak rob broers strand vergelijken
daartoe griekse staatssecretaris lijf kosovo cheer broek hout stijgt toezicht
dossiers fors voordelen e-mail zichtbaar regime @ sowieso jury papieren koude
melden ontwerp verkozen h. toeval dikwijls doorgaans namens verrassing
bevestigd marokkaanse applaus beloften cup geheime vluchtelingen toegepast
leeuw verschijnt schaal australië geïnteresseerd effectief danken armoede von
scherp gewonden wetten voorts automatisch massaal vrezen bord stroom houten
jongere zanger inclusief alcohol uitgesproken ruime cm geconfronteerd stelling
fusie geslacht verlopen pakistan plant zuid-afrika vooruitgang verhogen knie
aparte gepleegd
""".split())

# 1000 MFW German; http://www.wortschatz.uni-leipzig.de/Papers/top1000de.txt
basicwords_de = frozenset("""
der die und in den von zu das mit sich des auf für ist im dem nicht ein Die
eine als auch es an werden aus er hat daß sie nach wird bei einer Der um am
sind noch wie einem über einen Das so Sie zum war haben nur oder aber vor zur
bis mehr durch man sein wurde sei In Prozent hatte kann gegen vom können schon
wenn habe seine Mark ihre dann unter wir soll ich eines Es Jahr zwei Jahren
diese dieser wieder keine Uhr seiner worden Und will zwischen Im immer
Millionen Ein was sagte Er gibt alle DM diesem seit muß wurden beim doch jetzt
waren drei Jahre Mit neue neuen damit bereits da Auch ihr seinen müssen ab
ihrer Nach ohne sondern selbst ersten nun etwa Bei heute ihren weil ihm seien
Menschen Deutschland anderen werde Ich sagt Wir Eine rund Für Aber ihn Ende
jedoch Zeit sollen ins Wenn So seinem uns Stadt geht Doch sehr hier ganz erst
wollen Berlin vor allem sowie hatten kein deutschen machen lassen Als
Unternehmen andere ob dieses steht dabei wegen weiter denn beiden einmal etwas
Wie nichts allerdings vier gut viele wo viel dort alles Auf wäre SPD kommt
vergangenen denen fast fünf könnte nicht nur hätten Frau Am dafür kommen diesen
letzten zwar Diese großen dazu Von Mann Da sollte würde also bisher Leben
Milliarden Welt Regierung konnte ihrem Frauen während Land zehn würden stehen
ja USA heißt dies zurück Kinder dessen ihnen deren sogar Frage gewesen erste
gab liegt gar davon gestern geben Teil Polizei dass hätte eigenen kaum sieht
große Denn weitere Was sehen macht Angaben weniger gerade läßt Geld München
deutsche allen darauf wohl später könne deshalb aller kam Arbeit mich gegenüber
nächsten bleibt wenig lange gemacht Wer Dies Fall mir gehen Berliner mal Weg
CDU wollte sechs keinen Woche dagegen alten möglich gilt erklärte müsse Dabei
könnten Geschichte zusammen finden Tag Art erhalten Man Dollar Wochen jeder nie
bleiben besonders Jahres Deutschen Den Zu zunächst derzeit allein deutlich
Entwicklung weiß einige sollten Präsident geworden statt Bonn Platz inzwischen
Nur Freitag Um pro seines Damit Montag Europa schließlich Sonntag einfach
gehört eher oft Zahl neben hält weit Partei meisten Thema zeigt Politik Aus
zweiten Januar insgesamt je mußte Anfang hinter ebenfalls ging Mitarbeiter
darüber vielen Ziel darf Seite fest hin erklärt Namen Haus An Frankfurt
Gesellschaft Mittwoch damals Dienstag Hilfe Mai Markt Seit Tage Donnerstag
halten gleich nehmen solche Entscheidung besser alte Leute Ergebnis Samstag Daß
sagen System März tun Monaten kleinen lang Nicht knapp bringen wissen Kosten
Erfolg bekannt findet daran künftig wer acht Grünen schnell Grund scheint
Zukunft Stuttgart bin liegen politischen Gruppe Rolle stellt Juni sieben
September nämlich Männer Oktober Mrd überhaupt eigene Dann gegeben Außerdem
Stunden eigentlich Meter ließ Probleme vielleicht ebenso Bereich zum Beispiel
Bis Höhe Familie Während Bild Ländern Informationen Frankreich Tagen schwer
zuvor Vor genau April stellen neu erwartet Hamburg sicher führen Mal Über
mehrere Wirtschaft Mio Programm offenbar Hier weiteren natürlich konnten stark
Dezember Juli ganze kommenden Kunden bekommen eben kleine trotz wirklich Lage
Länder leicht gekommen Spiel laut November kurz politische führt innerhalb
unsere meint immer wieder Form Münchner AG anders ihres völlig beispielsweise
gute bislang August Hand jede GmbH Film Minuten erreicht beide Musik Kritik
Mitte Verfügung Buch dürfen Unter jeweils einigen Zum Umsatz spielen Daten
welche müßten hieß paar nachdem Kunst Euro gebracht Problem Noch jeden Ihre
Sprecher recht erneut längst europäischen Sein Eltern Beginn besteht Seine
mindestens machte Jetzt bietet außerdem Bürger Trainer bald Deutsche Schon
Fragen klar Durch Seiten gehören Dort erstmals Februar zeigen Titel Stück
größten FDP setzt Wert Frankfurter Staat möchte daher wolle Bundesregierung
lediglich Nacht Krieg Opfer Tod nimmt Firma zuletzt Werk hohen leben unter
anderem Dieser Kirche weiterhin gebe gestellt Mitglieder Rahmen zweite Paris
Situation gefunden Wochenende internationalen Wasser Recht sonst stand Hälfte
Möglichkeit versucht blieb junge Mehrheit Straße Sache arbeiten Monate Mutter
berichtet letzte Gericht wollten Ihr zwölf zumindest Wahl genug Weise Vater
Bericht amerikanischen hoch beginnt Wort obwohl Kopf spielt Interesse Westen
verloren Preis Erst jedem erreichen setzen spricht früher teilte Landes zudem
einzelnen bereit Blick Druck Bayern Kilometer gemeinsam Bedeutung Chance
Politiker Dazu Zwei besten Ansicht endlich Stelle direkt Beim Bevölkerung Viele
solchen Alle solle jungen Einsatz richtig größte sofort neuer ehemaligen
unserer dürfte schaffen Augen Rußland Internet Allerdings Raum Mannschaft neun
kamen Ausstellung Zeiten Dem einzige meine Nun Verfahren Angebot Richtung
Projekt niemand Kampf weder tatsächlich Personen dpa Heute geführt Gespräch
Kreis Hamburger Schule guten Hauptstadt durchaus Zusammenarbeit darin Amt
Schritt meist groß zufolge Sprache Region Punkte Vergleich genommen gleichen du
Ob Soldaten Universität verschiedenen Kollegen neues Bürgermeister Angst
stellte Sommer danach anderer gesagt Sicherheit Macht Bau handelt Folge Bilder
lag Osten Handel sprach Aufgabe Chef frei dennoch DDR hohe Firmen bzw Koalition
Mädchen Zur entwickelt fand Diskussion bringt Deshalb Hause Gefahr per zugleich
früheren dadurch ganzen abend erzählt Streit Vergangenheit Parteien
Verhandlungen jedenfalls gesehen französischen Trotz darunter Spieler forderte
Beispiel Meinung wenigen Publikum sowohl meinte mag Auto Lösung Boden Einen
Präsidenten hinaus Zwar verletzt weltweit Sohn bevor Peter mußten keiner
Produktion Ort braucht Zusammenhang Kind Verein sprechen Aktien gleichzeitig
London sogenannten Richter geplant Italien Mittel her freilich Mensch großer
Bonner wenige öffentlichen Unterstützung dritten nahm Bundesrepublik
Arbeitsplätze bedeutet Feld Dr. Bank oben gesetzt Ausland Ministerpräsident
Vertreter z.B. jedes ziehen Parlament berichtete Dieses China aufgrund Stellen
warum Kindern heraus heutigen Anteil Herr Öffentlichkeit Abend Selbst Liebe
Neben rechnen fällt New York Industrie WELT Stuttgarter wären Vorjahr Sicht
Idee Banken verlassen Leiter Bühne insbesondere offen stets Theater ändern
entschieden Staaten Experten Gesetz Geschäft Tochter angesichts gelten Mehr
erwarten läuft fordert Japan Sieg Ist Stimmen wählen russischen gewinnen CSU
bieten Nähe jährlich Bremen Schüler Rede Funktion Zuschauer hingegen anderes
Führung Besucher Drittel Moskau immerhin Vorsitzende Urteil Schließlich Kultur
betonte mittlerweile Saison Konzept suchen Zahlen Roman Gewalt Köln gesamte
indem EU Stunde ehemalige Auftrag entscheiden genannt tragen Börse langen
häufig Chancen Vor allem Position alt Luft Studenten übernehmen stärker ohnehin
zeigte geplanten Reihe darum verhindern begann Medien verkauft Minister wichtig
amerikanische sah gesamten einst verwendet vorbei Behörden helfen Folgen
bezeichnet
""".lower().split())

LANGDATA = dict(
		en=dict(
			syllables=countsyllables_en,
			words=words_en,
			beginnings=beginnings_en,
			basicwords=basicwords_en),
		nl=dict(
			syllables=countsyllables_nlde,
			words=words_nl,
			beginnings=beginnings_nl,
			basicwords=basicwords_nl),
		de=dict(
			syllables=countsyllables_nlde,
			words=words_de,
			beginnings=beginnings_de,
			basicwords=basicwords_de),
		)



# -*- coding: utf-8 -*-
# @Time    : 2018/8/1 9:49
# @Author  : yangyong
# @Email   : yangyong@findourlove.com
# @File    : three.py
# @Software: PyCharm
def raise_with_traceback(exc_type, traceback, *args, **kwargs):
    """
    用现有的“traceback”来提出“exc类型”的新异常。
    """
    raise exc_type(*args, **kwargs).with_traceback(traceback)


# -*- coding: utf-8 -*-
# @Time    : 2018/8/1 9:42
# @Author  : yangyong
# @Email   : yangyong@findourlove.com
# @File    : encoding.py
# @Software: PyCharm
import re
import chardet
import sys


RE_CHARSET = re.compile(br'<meta.*?charset=["\']*(.+?)["\'>]', flags=re.I)
RE_PRAGMA = re.compile(br'<meta.*?content=["\']*;?charset=(.+?)["\'>]', flags=re.I)
RE_XML = re.compile(br'^<\?xml.*?encoding=["\']*(.+?)["\'>]')

CHARSETS = {
    'big5': 'big5hkscs',
    'gb2312': 'gb18030',
    'ascii': 'utf-8',
    'maccyrillic': 'cp1251',
    'win1251': 'cp1251',
    'win-1251': 'cp1251',
    'windows-1251': 'cp1251',
}

def fix_charset(encoding):
    encoding = encoding.lower()
    return CHARSETS.get(encoding, encoding)


def get_encoding(page):
    # Regex for XML and HTML Meta charset declaration
    declared_encodings = (RE_CHARSET.findall(page) +
            RE_PRAGMA.findall(page) +
            RE_XML.findall(page))

    # Try any declared encodings
    for declared_encoding in declared_encodings:
        try:
            if sys.version_info[0] == 3:
                # declared_encoding will actually be bytes but .decode() only
                # accepts `str` type. Decode blindly with ascii because no one should
                # ever use non-ascii characters in the name of an encoding.
                declared_encoding = declared_encoding.decode('ascii', 'replace')

            encoding = fix_charset(declared_encoding)

            # Now let's decode the page
            page.decode()
            # It worked!
            return encoding
        except UnicodeDecodeError:
            pass

    # Fallback to chardet if declared encodings fail
    # Remove all HTML tags, and leave only text for chardet
    text = re.sub(b'(\s*</?[^>]*>)+\s*', b' ', page).strip()
    enc = 'utf-8'
    if len(text) < 10:
        return enc # can't guess
    res = chardet.detect(text)
    enc = res['encoding'] or 'utf-8'
    #print '->', enc, "%.2f" % res['confidence']
    enc = fix_charset(enc)
    return enc


# -*- coding: utf-8 -*-
# @Time    : 2018/8/1 9:46
# @Author  : yangyong
# @Email   : yangyong@findourlove.com
# @File    : readability.py
# @Software: PyCharm
from __future__ import print_function
import logging
import re
import sys

from collections import defaultdict
from lxml.etree import tostring
from lxml.etree import tounicode
from lxml.html import document_fromstring
from lxml.html import fragment_fromstring

from .cleaners import clean_attributes
from .cleaners import html_cleaner
from .htmls import build_doc
from .htmls import get_body
from .htmls import get_title
from .htmls import shorten_title
from .compat import str_
from .debug import describe, text_content

log = logging.getLogger("readability.readability")

REGEXES = {
    'unlikelyCandidatesRe': re.compile(
        'combx|comment|community|disqus|extra|foot|header|menu|remark|rss|shoutbox|sidebar|sponsor|ad-break|agegate|pagination|pager|popup|tweet|twitter',
        re.I),
    'okMaybeItsACandidateRe': re.compile('and|article|body|column|main|shadow', re.I),
    'positiveRe': re.compile('article|body|content|entry|hentry|main|page|pagination|post|text|blog|story', re.I),
    'negativeRe': re.compile(
        'combx|comment|com-|contact|foot|footer|footnote|masthead|media|meta|outbrain|promo|related|scroll|shoutbox|sidebar|sponsor|shopping|tags|tool|widget',
        re.I),
    'divToPElementsRe': re.compile('<(a|blockquote|dl|div|img|ol|p|pre|table|ul)', re.I),
    # 'replaceBrsRe': re.compile('(<br[^>]*>[ \n\r\t]*){2,}',re.I),
    # 'replaceFontsRe': re.compile('<(\/?)font[^>]*>',re.I),
    # 'trimRe': re.compile('^\s+|\s+$/'),
    # 'normalizeRe': re.compile('\s{2,}/'),
    # 'killBreaksRe': re.compile('(<br\s*\/?>(\s|&nbsp;?)*){1,}/'),
    'videoRe': re.compile('https?:\/\/(www\.)?(youtube|vimeo)\.com', re.I),
    # skipFootnoteLink:      /^\s*(\[?[a-z0-9]{1,2}\]?|^|edit|citation needed)\s*$/i,
}


class Unparseable(ValueError):
    pass


def to_int(x):
    if not x:
        return None
    x = x.strip()
    if x.endswith('px'):
        return int(x[:-2])
    if x.endswith('em'):
        return int(x[:-2]) * 12
    return int(x)


def clean(text):
    text = re.sub('\s*\n\s*', '\n', text)
    text = re.sub('\t|[ \t]{2,}', ' ', text)
    return text.strip()


def text_length(i):
    return len(clean(i.text_content() or ""))


regexp_type = type(re.compile('hello, world'))


def compile_pattern(elements):
    if not elements:
        return None
    elif isinstance(elements, (list, tuple)):
        return list(elements)
    elif isinstance(elements, regexp_type):
        return elements
    else:
        # assume string or string like object
        elements = elements.split(',')
        return re.compile(u'|'.join([re.escape(x.lower()) for x in elements]), re.U)


class Document:

    def __init__(self, input, positive_keywords=None, negative_keywords=None,
                 url=None, min_text_length=25, retry_length=250, xpath=False):
        self.input = input
        self.html = None
        self.encoding = None
        self.positive_keywords = compile_pattern(positive_keywords)
        self.negative_keywords = compile_pattern(negative_keywords)
        self.url = url
        self.min_text_length = min_text_length
        self.retry_length = retry_length
        self.xpath = xpath

    def _html(self, force=False):
        if force or self.html is None:
            self.html = self._parse(self.input)
            if self.xpath:
                root = self.html.getroottree()
                for i in self.html.getiterator():
                    # print root.getpath(i)
                    i.attrib['x'] = root.getpath(i)
        return self.html

    def _parse(self, input):
        doc, self.encoding = build_doc(input)
        doc = html_cleaner.clean_html(doc)
        base_href = self.url
        if base_href:
            # trying to guard against bad links like <a href="http://[http://...">
            try:
                # such support is added in lxml 3.3.0
                doc.make_links_absolute(base_href, resolve_base_href=True, handle_failures='discard')
            except TypeError:  # make_links_absolute() got an unexpected keyword argument 'handle_failures'
                # then we have lxml < 3.3.0
                # please upgrade to lxml >= 3.3.0 if you're failing here!
                doc.make_links_absolute(base_href, resolve_base_href=True)
        else:
            doc.resolve_base_href()
        return doc

    def content(self):
        return get_body(self._html(True))

    def title(self):
        return get_title(self._html(True))

    def short_title(self):
        return shorten_title(self._html(True))

    def get_clean_html(self):
        return clean_attributes(tounicode(self.html))

    def summary(self, html_partial=False):
        try:
            ruthless = True
            while True:
                self._html(True)
                for i in self.tags(self.html, 'script', 'style'):
                    i.drop_tree()
                for i in self.tags(self.html, 'body'):
                    i.set('id', 'readabilityBody')
                if ruthless:
                    self.remove_unlikely_candidates()
                self.transform_misused_divs_into_paragraphs()
                candidates = self.score_paragraphs()

                best_candidate = self.select_best_candidate(candidates)

                if best_candidate:
                    article = self.get_article(candidates, best_candidate,
                                               html_partial=html_partial)
                else:
                    if ruthless:
                        log.info("ruthless removal did not work. ")
                        ruthless = False
                        log.debug(
                            ("ended up stripping too much - "
                             "going for a safer _parse"))
                        # try again
                        continue
                    else:
                        log.debug(
                            ("Ruthless and lenient parsing did not work. "
                             "Returning raw html"))
                        article = self.html.find('body')
                        if article is None:
                            article = self.html
                cleaned_article = self.sanitize(article, candidates)

                article_length = len(cleaned_article or '')
                retry_length = self.retry_length
                of_acceptable_length = article_length >= retry_length
                if ruthless and not of_acceptable_length:
                    ruthless = False
                    # Loop through and try again.
                    continue
                else:
                    return cleaned_article
        except Exception as e:
            log.exception('error getting summary: ')
            if sys.version_info[0] == 2:
                from .compat.two import raise_with_traceback
            else:
                from .compat.three import raise_with_traceback
            raise_with_traceback(Unparseable, sys.exc_info()[2], str_(e))

    def get_article(self, candidates, best_candidate, html_partial=False):
        # Now that we have the top candidate, look through its siblings for
        # content that might also be related.
        # Things like preambles, content split by ads that we removed, etc.
        sibling_score_threshold = max([
            10,
            best_candidate['content_score'] * 0.2])
        # create a new html document with a html->body->div
        if html_partial:
            output = fragment_fromstring('<div/>')
        else:
            output = document_fromstring('<div/>')
        best_elem = best_candidate['elem']
        parent = best_elem.getparent()
        siblings = parent.getchildren() if parent is not None else [best_elem]
        for sibling in siblings:
            # in lxml there no concept of simple text
            # if isinstance(sibling, NavigableString): continue
            append = False
            if sibling is best_elem:
                append = True
            sibling_key = sibling  # HashableElement(sibling)
            if sibling_key in candidates and \
                    candidates[sibling_key]['content_score'] >= sibling_score_threshold:
                append = True

            if sibling.tag == "p":
                link_density = self.get_link_density(sibling)
                node_content = sibling.text or ""
                node_length = len(node_content)

                if node_length > 80 and link_density < 0.25:
                    append = True
                elif node_length <= 80 \
                        and link_density == 0 \
                        and re.search('\.( |$)', node_content):
                    append = True

            if append:
                # We don't want to append directly to output, but the div
                # in html->body->div
                if html_partial:
                    output.append(sibling)
                else:
                    output.getchildren()[0].getchildren()[0].append(sibling)
        # if output is not None:
        #    output.append(best_elem)
        return output

    def select_best_candidate(self, candidates):
        if not candidates:
            return None

        sorted_candidates = sorted(
            candidates.values(),
            key=lambda x: x['content_score'],
            reverse=True
        )
        for candidate in sorted_candidates[:5]:
            elem = candidate['elem']
            log.info("Top 5 : %6.3f %s" % (
                candidate['content_score'],
                describe(elem)))

        best_candidate = sorted_candidates[0]
        return best_candidate

    def get_link_density(self, elem):
        link_length = 0
        for i in elem.findall(".//a"):
            link_length += text_length(i)
        # if len(elem.findall(".//div") or elem.findall(".//p")):
        #    link_length = link_length
        total_length = text_length(elem)
        return float(link_length) / max(total_length, 1)

    def score_paragraphs(self, ):
        MIN_LEN = self.min_text_length
        candidates = {}
        ordered = []
        for elem in self.tags(self._html(), "p", "pre", "td"):
            parent_node = elem.getparent()
            if parent_node is None:
                continue
            grand_parent_node = parent_node.getparent()

            inner_text = clean(elem.text_content() or "")
            inner_text_len = len(inner_text)

            # If this paragraph is less than 25 characters
            # don't even count it.
            if inner_text_len < MIN_LEN:
                continue

            if parent_node not in candidates:
                candidates[parent_node] = self.score_node(parent_node)
                ordered.append(parent_node)

            if grand_parent_node is not None and grand_parent_node not in candidates:
                candidates[grand_parent_node] = self.score_node(
                    grand_parent_node)
                ordered.append(grand_parent_node)

            content_score = 1
            content_score += len(inner_text.split(','))
            content_score += min((inner_text_len / 100), 3)
            # if elem not in candidates:
            #    candidates[elem] = self.score_node(elem)

            # WTF? candidates[elem]['content_score'] += content_score
            candidates[parent_node]['content_score'] += content_score
            if grand_parent_node is not None:
                candidates[grand_parent_node]['content_score'] += content_score / 2.0

        # Scale the final candidates score based on link density. Good content
        # should have a relatively small link density (5% or less) and be
        # mostly unaffected by this operation.
        for elem in ordered:
            candidate = candidates[elem]
            ld = self.get_link_density(elem)
            score = candidate['content_score']
            log.debug("Branch %6.3f %s link density %.3f -> %6.3f" % (
                score,
                describe(elem),
                ld,
                score * (1 - ld)))
            candidate['content_score'] *= (1 - ld)

        return candidates

    def class_weight(self, e):
        weight = 0
        for feature in [e.get('class', None), e.get('id', None)]:
            if feature:
                if REGEXES['negativeRe'].search(feature):
                    weight -= 25

                if REGEXES['positiveRe'].search(feature):
                    weight += 25

                if self.positive_keywords and self.positive_keywords.search(feature):
                    weight += 25

                if self.negative_keywords and self.negative_keywords.search(feature):
                    weight -= 25

        if self.positive_keywords and self.positive_keywords.match('tag-' + e.tag):
            weight += 25

        if self.negative_keywords and self.negative_keywords.match('tag-' + e.tag):
            weight -= 25

        return weight

    def score_node(self, elem):
        content_score = self.class_weight(elem)
        name = elem.tag.lower()
        if name == "div":
            content_score += 5
        elif name in ["pre", "td", "blockquote"]:
            content_score += 3
        elif name in ["address", "ol", "ul", "dl", "dd", "dt", "li", "form"]:
            content_score -= 3
        elif name in ["h1", "h2", "h3", "h4", "h5", "h6", "th"]:
            content_score -= 5
        return {
            'content_score': content_score,
            'elem': elem
        }

    def remove_unlikely_candidates(self):
        for elem in self.html.iter():
            s = "%s %s" % (elem.get('class', ''), elem.get('id', ''))
            if len(s) < 2:
                continue
            if REGEXES['unlikelyCandidatesRe'].search(s) and (
            not REGEXES['okMaybeItsACandidateRe'].search(s)) and elem.tag not in ['html', 'body']:
                log.debug("Removing unlikely candidate - %s" % describe(elem))
                elem.drop_tree()

    def transform_misused_divs_into_paragraphs(self):
        for elem in self.tags(self.html, 'div'):

            if not REGEXES['divToPElementsRe'].search(
                    str_(b''.join(map(tostring, list(elem))))):
                # log.debug("Altering %s to p" % (describe(elem)))
                elem.tag = "p"
                # print "Fixed element "+describe(elem)

        for elem in self.tags(self.html, 'div'):
            if elem.text and elem.text.strip():
                p = fragment_fromstring('<p/>')
                p.text = elem.text
                elem.text = None
                elem.insert(0, p)
                # print "Appended "+tounicode(p)+" to "+describe(elem)

            for pos, child in reversed(list(enumerate(elem))):
                if child.tail and child.tail.strip():
                    p = fragment_fromstring('<p/>')
                    p.text = child.tail
                    child.tail = None
                    elem.insert(pos + 1, p)
                    # print "Inserted "+tounicode(p)+" to "+describe(elem)
                if child.tag == 'br':
                    # print 'Dropped <br> at '+describe(elem)
                    child.drop_tree()

    def tags(self, node, *tag_names):
        for tag_name in tag_names:
            for e in node.findall('.//%s' % tag_name):
                yield e

    def reverse_tags(self, node, *tag_names):
        for tag_name in tag_names:
            for e in reversed(node.findall('.//%s' % tag_name)):
                yield e

    def sanitize(self, node, candidates):
        MIN_LEN = self.min_text_length
        for header in self.tags(node, "h1", "h2", "h3", "h4", "h5", "h6"):
            if self.class_weight(header) < 0 or self.get_link_density(header) > 0.33:
                header.drop_tree()

        for elem in self.tags(node, "form", "textarea"):
            elem.drop_tree()

        for elem in self.tags(node, "iframe"):
            if "src" in elem.attrib and REGEXES["videoRe"].search(elem.attrib["src"]):
                elem.text = "VIDEO"  # ADD content to iframe text node to force <iframe></iframe> proper output
            else:
                elem.drop_tree()

        allowed = {}
        # Conditionally clean <table>s, <ul>s, and <div>s
        for el in self.reverse_tags(node, "table", "ul", "div"):
            if el in allowed:
                continue
            weight = self.class_weight(el)
            if el in candidates:
                content_score = candidates[el]['content_score']
                # print '!',el, '-> %6.3f' % content_score
            else:
                content_score = 0
            tag = el.tag

            if weight + content_score < 0:
                log.debug("Removed %s with score %6.3f and weight %-3s" %
                          (describe(el), content_score, weight,))
                el.drop_tree()
            elif el.text_content().count(",") < 10:
                counts = {}
                for kind in ['p', 'img', 'li', 'a', 'embed', 'input']:
                    counts[kind] = len(el.findall('.//%s' % kind))
                counts["li"] -= 100
                counts["input"] -= len(el.findall('.//input[@type="hidden"]'))

                # Count the text length excluding any surrounding whitespace
                content_length = text_length(el)
                link_density = self.get_link_density(el)
                parent_node = el.getparent()
                if parent_node is not None:
                    if parent_node in candidates:
                        content_score = candidates[parent_node]['content_score']
                    else:
                        content_score = 0
                to_remove = False
                reason = ""

                # if el.tag == 'div' and counts["img"] >= 1:
                #    continue
                if counts["p"] and counts["img"] > 1 + counts["p"] * 1.3:
                    reason = "too many images (%s)" % counts["img"]
                    to_remove = True
                elif counts["li"] > counts["p"] and tag != "ul" and tag != "ol":
                    reason = "more <li>s than <p>s"
                    to_remove = True
                elif counts["input"] > (counts["p"] / 3):
                    reason = "less than 3x <p>s than <input>s"
                    to_remove = True
                elif content_length < MIN_LEN and counts["img"] == 0:
                    reason = "too short content length %s without a single image" % content_length
                    to_remove = True
                elif content_length < MIN_LEN and counts["img"] > 2:
                    reason = "too short content length %s and too many images" % content_length
                    to_remove = True
                elif weight < 25 and link_density > 0.2:
                    reason = "too many links %.3f for its weight %s" % (
                        link_density, weight)
                    to_remove = True
                elif weight >= 25 and link_density > 0.5:
                    reason = "too many links %.3f for its weight %s" % (
                        link_density, weight)
                    to_remove = True
                elif (counts["embed"] == 1 and content_length < 75) or counts["embed"] > 1:
                    reason = "<embed>s with too short content length, or too many <embed>s"
                    to_remove = True
                elif not content_length:
                    reason = "no content"
                    to_remove = True


                    # find x non empty preceding and succeeding siblings
                    i, j = 0, 0
                    x = 1
                    siblings = []
                    for sib in el.itersiblings():
                        # log.debug(sib.text_content())
                        sib_content_length = text_length(sib)
                        if sib_content_length:
                            i = + 1
                            siblings.append(sib_content_length)
                            if i == x:
                                break
                    for sib in el.itersiblings(preceding=True):
                        # log.debug(sib.text_content())
                        sib_content_length = text_length(sib)
                        if sib_content_length:
                            j = + 1
                            siblings.append(sib_content_length)
                            if j == x:
                                break
                    # log.debug(str_(siblings))
                    if siblings and sum(siblings) > 1000:
                        to_remove = False
                        log.debug("Allowing %s" % describe(el))
                        for desnode in self.tags(el, "table", "ul", "div"):
                            allowed[desnode] = True

                if to_remove:
                    log.debug("Removed %6.3f %s with weight %s cause it has %s." %
                              (content_score, describe(el), weight, reason))
                    # print tounicode(el)
                    # log.debug("pname %s pweight %.3f" %(pname, pweight))
                    el.drop_tree()
                else:
                    log.debug("Not removing %s of length %s: %s" % (
                        describe(el), content_length, text_content(el)))

        self.html = node
        return self.get_clean_html()


def main():
    VERBOSITY = {
        1: logging.WARNING,
        2: logging.INFO,
        3: logging.DEBUG
    }

    from optparse import OptionParser
    parser = OptionParser(usage="%prog: [options] [file]")
    parser.add_option('-v', '--verbose', action='count', default=0)
    parser.add_option('-b', '--browser', default=None, action='store_true', help="open in browser")
    parser.add_option('-l', '--log', default=None, help="save logs into file (appended)")
    parser.add_option('-u', '--url', default=None, help="use URL instead of a local file")
    parser.add_option('-x', '--xpath', default=None, help="add original xpath")
    parser.add_option('-p', '--positive-keywords', default=None, help="positive keywords (separated with comma)",
                      action='store')
    parser.add_option('-n', '--negative-keywords', default=None, help="negative keywords (separated with comma)",
                      action='store')
    (options, args) = parser.parse_args()

    if options.verbose:
        logging.basicConfig(level=VERBOSITY[options.verbose], filename=options.log,
                            format='%(asctime)s: %(levelname)s: %(message)s (at %(filename)s: %(lineno)d)')

    if not (len(args) == 1 or options.url):
        parser.print_help()
        sys.exit(1)

    file = None
    if options.url:
        headers = {'User-Agent': 'Mozilla/5.0'}
        if sys.version_info[0] == 3:
            import urllib.request, urllib.parse, urllib.error
            request = urllib.request.Request(options.url, None, headers)
            file = urllib.request.urlopen(request)
        else:
            import urllib2
            request = urllib2.Request(options.url, None, headers)
            file = urllib2.urlopen(request)
    else:
        file = open(args[0], 'rt')
    try:
        doc = Document(file.read(),
                       url=options.url,
                       positive_keywords=options.positive_keywords,
                       negative_keywords=options.negative_keywords,
                       )
        if options.browser:
            from .browser import open_in_browser
            result = '<h2>' + doc.short_title() + '</h2><br/>' + doc.summary()
            open_in_browser(result)
        else:
            enc = sys.__stdout__.encoding or 'utf-8'
            result = 'Title:' + doc.short_title() + '\n' + doc.summary()
            if sys.version_info[0] == 3:
                print(result)
            else:
                print(result.encode(enc, 'replace'))
    finally:
        file.close()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# @Time    : 2018/8/1 9:41
# @Author  : yangyong
# @Email   : yangyong@findourlove.com
# @File    : debug.py
# @Software: PyCharm
import re

uids = {}
uids_document = None

def describe_node(node):
    global uids
    if node is None:
        return ''
    if not hasattr(node, 'tag'):
        return "[%s]" % type(node)
    name = node.tag
    if node.get('id', ''):
        name += '#' + node.get('id')
    if node.get('class', '').strip():
        name += '.' + '.'.join(node.get('class').split())
    if name[:4] in ['div#', 'div.']:
        name = name[3:]
    if name in ['tr', 'td', 'div', 'p']:
        uid = uids.get(node)
        if uid is None:
            uid = uids[node] = len(uids) + 1
        name += "{%02d}" % uid
    return name


def describe(node, depth=1):
    global uids, uids_document
    doc = node.getroottree().getroot()
    if doc != uids_document:
        uids = {}
        uids_document = doc

    #return repr(NodeRepr(node))
    parent = ''
    if depth and node.getparent() is not None:
        parent = describe(node.getparent(), depth=depth - 1) + '>'
    return parent + describe_node(node)


RE_COLLAPSE_WHITESPACES = re.compile('\s+', re.U)


def text_content(elem, length=40):
    content = RE_COLLAPSE_WHITESPACES.sub(' ', elem.text_content().replace('\r', ''))
    if len(content) < length:
        return content
    return content[:length] + '...'



# -*- coding: utf-8 -*-
# @Time    : 2018/8/1 9:43
# @Author  : yangyong
# @Email   : yangyong@findourlove.com
# @File    : htmls.py
# @Software: PyCharm
from lxml.html import tostring
import logging
import lxml.html
import re, sys

from .cleaners import normalize_spaces, clean_attributes
from .encoding import get_encoding
from .compat import str_

utf8_parser = lxml.html.HTMLParser(encoding='utf-8')


def build_doc(page):
    if isinstance(page, str_):
        encoding = None
        decoded_page = page
    else:
        encoding = get_encoding(page) or 'utf-8'
        decoded_page = page.decode(encoding, 'replace')

    doc = lxml.html.document_fromstring(decoded_page.encode('utf-8', 'replace'), parser=utf8_parser)
    return doc, encoding


def js_re(src, pattern, flags, repl):
    return re.compile(pattern, flags).sub(src, repl.replace('$', '\\'))


def normalize_entities(cur_title):
    entities = {
        u'\u2014': '-',
        u'\u2013': '-',
        u'&mdash;': '-',
        u'&ndash;': '-',
        u'\u00A0': ' ',
        u'\u00AB': '"',
        u'\u00BB': '"',
        u'&quot;': '"',
    }
    for c, r in entities.items():
        if c in cur_title:
            cur_title = cur_title.replace(c, r)

    return cur_title


def norm_title(title):
    return normalize_entities(normalize_spaces(title))


def get_title(doc):
    title = doc.find('.//title')
    if title is None or title.text is None or len(title.text) == 0:
        return '[no-title]'

    return norm_title(title.text)


def add_match(collection, text, orig):
    text = norm_title(text)
    if len(text.split()) >= 2 and len(text) >= 15:
        if text.replace('"', '') in orig.replace('"', ''):
            collection.add(text)


TITLE_CSS_HEURISTICS = ['#title', '#head', '#heading', '.pageTitle',
                        '.news_title', '.title', '.head', '.heading',
                        '.contentheading', '.small_header_red']


def shorten_title(doc):
    title = doc.find('.//title')
    if title is None or title.text is None or len(title.text) == 0:
        return ''

    title = orig = norm_title(title.text)

    candidates = set()

    for item in ['.//h1', './/h2', './/h3']:
        for e in list(doc.iterfind(item)):
            if e.text:
                add_match(candidates, e.text, orig)
            if e.text_content():
                add_match(candidates, e.text_content(), orig)

    for item in TITLE_CSS_HEURISTICS:
        for e in doc.cssselect(item):
            if e.text:
                add_match(candidates, e.text, orig)
            if e.text_content():
                add_match(candidates, e.text_content(), orig)

    if candidates:
        title = sorted(candidates, key=len)[-1]
    else:
        for delimiter in [' | ', ' - ', ' :: ', ' / ']:
            if delimiter in title:
                parts = orig.split(delimiter)
                if len(parts[0].split()) >= 4:
                    title = parts[0]
                    break
                elif len(parts[-1].split()) >= 4:
                    title = parts[-1]
                    break
        else:
            if ': ' in title:
                parts = orig.split(': ')
                if len(parts[-1].split()) >= 4:
                    title = parts[-1]
                else:
                    title = orig.split(': ', 1)[1]

    if not 15 < len(title) < 150:
        return orig

    return title


def get_body(doc):
    for elem in doc.xpath('.//script | .//link | .//style'):
        elem.drop_tree()
    # tostring() always return utf-8 encoded string
    raw_html = str_(tostring(doc.body or doc))
    cleaned = clean_attributes(raw_html)
    try:
        return cleaned
    except Exception:
        # logging.error("cleansing broke html content: %s\n---------\n%s" % (raw_html, cleaned))
        return raw_html


# -*- coding: utf-8 -*-
# @Time    : 2018/7/30 13:21
# @Author  : yangyong
# @Email   : yangyong@findourlove.com
# @File    : setup.py
# @Software: PyCharm
from __future__ import print_function
from setuptools import setup,find_packages
import sys

setup(
    name="extensive_collection",
    version="0.0.5",
    author="Edwin yang",
    author_email="yangyong@findourlove.com",
    description="extensive_collection",
    long_description=open("README.rst").read(),
    license="MIT",
    packages=['extensive_collection','extensive_collection/compat'],
    install_requires=[
        "pandas",
    ],
    classifiers=[
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Topic :: Text Processing :: Indexing",
        "Topic :: Utilities",
        "Topic :: Internet",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
    ],

)


# -*- coding: utf-8 -*-
# @Time    : 2018/8/1 9:39
# @Author  : yangyong
# @Email   : yangyong@findourlove.com
# @File    : browser.py
# @Software: PyCharm
def open_in_browser(html):
    """
    Open the HTML document in a web browser, saving it to a temporary
    file to open it.  Note that this does not delete the file after
    use.  This is mainly meant for debugging.
    """
    import os
    import webbrowser
    import tempfile
    handle, fn = tempfile.mkstemp(suffix='.html')
    f = os.fdopen(handle, 'wb')
    try:
        f.write(b"<meta charset='UTF-8' />")
        f.write(html.encode('utf-8'))
    finally:
        # we leak the file itself here, but we should at least close it
        f.close()
    url = 'file://' + fn.replace(os.path.sep, '/')
    webbrowser.open(url)
    return url